#version 330 core

#define highp

// Interpolated values from the vertex shaders
in vec2 UV;
in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in vec4  ShadowCoord;

// Ouput data
layout (location = 0) out vec3 color;

// Values that stay constant for the whole mesh.
uniform sampler2D textureIn;
uniform sampler2D textureIn2;
uniform sampler2D textureIn3;
uniform sampler2D textureIn4;
uniform mat4 MV;
//uniform vec3 LightPosition_worldspace;
uniform sampler2DShadow shadowTexture;
//uniform sampler2D shadowTexture;
uniform float time;

float snoise(vec3 v);
float snoise(vec2 v);

void main()
{
    float topMix = round(abs(snoise(Position_worldspace.xz/2000.0)+0.6)/2.0);
	// Material properties
    vec3 MaterialDiffuseColor = mix(texture(textureIn, UV).rgb,texture(textureIn3, UV).rgb, topMix);
    
	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
    
    float visibility = 1.0;
    if(Position_worldspace.y < -7) {
        //float c = (1-snoise(vec3(UV.xy*2.0,time)));
        //visibility *= clamp(c*c,0,1);//Position_worldspace.xy/50.0, time/4.0)));
        float c = abs(snoise(vec3(Position_worldspace.xz/200.0, time/4.0)));
        c = 1.0-clamp(c, 0.0, 0.6);
        visibility = visibility*0.7+c*0.3;
        MaterialDiffuseColor = MaterialDiffuseColor*0.9+vec3(0.0,0.3,0.5)*0.1;
    }
    
#define bias -0.05
    vec2 poissonDisk[4] = vec2[](
                                 vec2( -0.94201624, -0.39906216 ),
                                 vec2( 0.94558609, -0.76890725 ),
                                 vec2( -0.094184101, -0.92938870 ),
                                 vec2( 0.34495938, 0.29387760 )
                                 );
    for (int i=0;i<4;i++){
        if ( texture( shadowTexture, ShadowCoord.xyz + vec3(poissonDisk[i]/700.0,0) )  < ShadowCoord.z-bias ){
            visibility-=0.2;
        }
    }
    
#define ambientIntensity 0.5f
    float diffuseIntensity = clamp(dot(n,-l),0,1);
    color = MaterialDiffuseColor * (ambientIntensity + visibility * diffuseIntensity);
}


//uniform vec3 sunColor;
//uniform vec3 sunDirection;
vec3 sunLight(const vec3 surfaceNormal, const vec3 eyeNormal, float shiny, float spec, vec3 diffuse){
    vec3 sunColor = vec3(1,1,1);
    vec3 sunDirection = vec3(0.0f,-100.0f,-500.0f);
    
    vec3 diffuseColor = max(dot(sunDirection, surfaceNormal),0.0)*sunColor*diffuse;
    vec3 reflection = normalize(reflect(-sunDirection, surfaceNormal));
    float direction = max(0.0, dot(eyeNormal, reflection));
    vec3 specular = pow(direction, shiny)*sunColor*spec;
    return diffuseColor + specular;
}


void main3()
{
    #define bias -0.05
    float visibility = 1;
    
    vec2 poissonDisk[4] = vec2[](
                                 vec2( -0.94201624, -0.39906216 ),
                                 vec2( 0.94558609, -0.76890725 ),
                                 vec2( -0.094184101, -0.92938870 ),
                                 vec2( 0.34495938, 0.29387760 )
                                 );
    for (int i=0;i<4;i++){
        if ( texture( shadowTexture, ShadowCoord.xyz + vec3(poissonDisk[i]/700.0,0) )  < ShadowCoord.z-bias ){
            visibility-=0.2;
        }
    }
    
//    vec3 LightPosition_worldspace = vec3(0,5,5);
    vec3 LightPosition_worldspace = vec3(0.0f,100.0f,500.0f);
    
    float topMix = round((abs(snoise(Position_worldspace.xz/2000.0)+0.6)/2.0));
    vec3 MaterialDiffuseColor = mix(texture(textureIn, UV).rgb,texture(textureIn3, UV).rgb, topMix);
    vec3 MaterialNormal = normalize(2.0f*mix(texture(textureIn2, UV).rgb,texture(textureIn4, UV).rgb, topMix)-1.0f);
    
	color = visibility * sunLight(MaterialNormal, EyeDirection_cameraspace, 15.0, 2.5 , MaterialDiffuseColor);//1.0);
}

void main2()
{
//    vec3 LightPosition_worldspace = vec3(0,5,5);
    vec3 LightPosition_worldspace = vec3(0.0f,100.0f,500.0f);
    
    // Light emission properties
	// You probably want to put them as uniforms
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 1000.0f;
	
    float topMix = round((abs(snoise(Position_worldspace.xz/2000.0)+0.6)/2.0));
	// Material properties
    vec3 MaterialDiffuseColor = mix(texture(textureIn, UV).rgb,texture(textureIn3, UV).rgb, topMix);
    vec3 MaterialNormal = normalize(2.0f*mix(texture(textureIn2, UV).rgb,texture(textureIn4, UV).rgb, topMix)-1.0f);
    
//	vec3 MaterialDiffuseColor = mix(texture(textureIn, UV).rgb,texture(textureIn3, UV).rgb, step(Position_worldspace.y, -7));
//    vec3 MaterialNormal = mix(texture(textureIn2, UV).rgb,texture(textureIn4, UV).rgb, step(Position_worldspace.y, -6.5));
	vec3 MaterialAmbientColor = vec3(0.5,0.5,0.5) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.3,0.3,0.3);
    
	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );
    
	// Normal of the computed fragment, in camera space
//	vec3 n = normalize( MaterialNormal); //Normal_cameraspace );
    vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	// Cosine of the angle between the normal and the light direction,
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
    //    float bias = 0.005;
    //    float visibility = 1.0;
    //    if (texture(shadowTexture, ShadowCoord.xy).x  <  ShadowCoord.z-bias) {
    //        visibility = 0.5;
    //    }
#define bias -0.05
    
    float visibility = 1;//texture(shadowTexture, vec3(ShadowCoord.xy, ((ShadowCoord.z-0.0005)/ShadowCoord.w)));// * 0.6 + 0.4;
////    float visibility = texture(shadowTexture, ShadowCoord.xyz);// * 0.6 + 0.4;
//    if ( texture( shadowTexture, (ShadowCoord.xy/ShadowCoord.w) ).r  <  (ShadowCoord.z-bias)/ShadowCoord.w ) {
////        if ( textureProj( shadowTexture, ShadowCoord.xyw ).z  <  (ShadowCoord.z-bias)/ShadowCoord.w ) {
//        visibility = 0.5;
//    }
    
    vec2 poissonDisk[4] = vec2[](
                                 vec2( -0.94201624, -0.39906216 ),
                                 vec2( 0.94558609, -0.76890725 ),
                                 vec2( -0.094184101, -0.92938870 ),
                                 vec2( 0.34495938, 0.29387760 )
                                 );
    for (int i=0;i<4;i++){
        if ( texture( shadowTexture, ShadowCoord.xyz + vec3(poissonDisk[i]/700.0,0) )  < ShadowCoord.z-bias ){
            visibility-=0.2;
        }
    }
    
//    //  - A random sample, based on the pixel's screen location.
//    //    No banding, but the shadow moves with the camera, which looks weird.
//    int index = int(16.0*random(gl_FragCoord.xyy, i))%16;
//    //  - A random sample, based on the pixel's position in world space.
//    //    The position is rounded to the millimeter to avoid too much aliasing
//    //int index = int(16.0*random(floor(Position_worldspace.xyz*1000.0), i))%16;
//
//    float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
//    return fract(sin(dot_product) * 43758.5453);
//    for (int i=0;i<4;i++){
//        int index = // A random number between 0 and 15, different for each pixel (and each i !)
//        visibility -= 0.2*(1.0-texture( shadowTexture, vec3(ShadowCoord.xy + poissonDisk[index]/700.0,  (ShadowCoord.z-bias)/ShadowCoord.w) ));
//    }

    
//    if(Position_worldspace.y < -7) {
//        //float c = (1-snoise(vec3(UV.xy*2.0,time)));
//        //visibility *= clamp(c*c,0,1);//Position_worldspace.xy/50.0, time/4.0)));
//        float c = abs(snoise(vec3(Position_worldspace.xz/200.0, time/4.0)));
//        c = 1.0-clamp(c, 0.0, 0.6);
//        visibility = visibility*0.8+c*0.2;
//        MaterialAmbientColor = MaterialAmbientColor*0.9+vec3(0.0,0.3,0.5)*0.1;
//        MaterialDiffuseColor = MaterialDiffuseColor*0.9+vec3(0.0,0.3,0.5)*0.1;
//    }
    
    //    if(visibility < 0.1) color = vec3(1, 0, 0);
    //    else if(visibility < 0.5) color = vec3(0, 1, 0);
    //    else color = vec3(0,0,1);
    
	color = //vec3(visibility,visibility,visibility);
    // Ambient : simulates indirect lighting
    visibility * MaterialAmbientColor +
    // Diffuse : "color" of the object
    visibility * MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance) +
    // Specular : reflective highlight, like a mirror
    visibility * MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) / (distance*distance);
}

//////////////////////// INCLUDED /////////////////
//
// Description : Array and textureless GLSL 2D/3D/4D simplex
//               noise functions.
//      Author : Ian McEwan, Ashima Arts.
//  Maintainer : ijm
//     Lastmod : 20110822 (ijm)
//     License : Copyright (C) 2011 Ashima Arts. All rights reserved.
//               Distributed under the MIT License. See LICENSE file.
//               https://github.com/ashima/webgl-noise
//

vec3 mod289(vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 mod289(vec4 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 permute(vec4 x) {
    return mod289(((x*34.0)+1.0)*x);
}

vec4 taylorInvSqrt(vec4 r)
{
    return 1.79284291400159 - 0.85373472095314 * r;
}

float snoise(vec3 v)
{
    const vec2  C = vec2(1.0/6.0, 1.0/3.0) ;
    const vec4  D = vec4(0.0, 0.5, 1.0, 2.0);
    
    // First corner
    vec3 i  = floor(v + dot(v, C.yyy) );
    vec3 x0 =   v - i + dot(i, C.xxx) ;
    
    // Other corners
    vec3 g = step(x0.yzx, x0.xyz);
    vec3 l = 1.0 - g;
    vec3 i1 = min( g.xyz, l.zxy );
    vec3 i2 = max( g.xyz, l.zxy );
    
    //   x0 = x0 - 0.0 + 0.0 * C.xxx;
    //   x1 = x0 - i1  + 1.0 * C.xxx;
    //   x2 = x0 - i2  + 2.0 * C.xxx;
    //   x3 = x0 - 1.0 + 3.0 * C.xxx;
    vec3 x1 = x0 - i1 + C.xxx;
    vec3 x2 = x0 - i2 + C.yyy; // 2.0*C.x = 1/3 = C.y
    vec3 x3 = x0 - D.yyy;      // -1.0+3.0*C.x = -0.5 = -D.y
    
    // Permutations
    i = mod289(i);
    vec4 p = permute( permute( permute(
                                       i.z + vec4(0.0, i1.z, i2.z, 1.0 ))
                              + i.y + vec4(0.0, i1.y, i2.y, 1.0 ))
                     + i.x + vec4(0.0, i1.x, i2.x, 1.0 ));
    
    // Gradients: 7x7 points over a square, mapped onto an octahedron.
    // The ring size 17*17 = 289 is close to a multiple of 49 (49*6 = 294)
    float n_ = 0.142857142857; // 1.0/7.0
    vec3  ns = n_ * D.wyz - D.xzx;
    
    vec4 j = p - 49.0 * floor(p * ns.z * ns.z);  //  mod(p,7*7)
    
    vec4 x_ = floor(j * ns.z);
    vec4 y_ = floor(j - 7.0 * x_ );    // mod(j,N)
    
    vec4 x = x_ *ns.x + ns.yyyy;
    vec4 y = y_ *ns.x + ns.yyyy;
    vec4 h = 1.0 - abs(x) - abs(y);
    
    vec4 b0 = vec4( x.xy, y.xy );
    vec4 b1 = vec4( x.zw, y.zw );
    
    //vec4 s0 = vec4(lessThan(b0,0.0))*2.0 - 1.0;
    //vec4 s1 = vec4(lessThan(b1,0.0))*2.0 - 1.0;
    vec4 s0 = floor(b0)*2.0 + 1.0;
    vec4 s1 = floor(b1)*2.0 + 1.0;
    vec4 sh = -step(h, vec4(0.0));
    
    vec4 a0 = b0.xzyw + s0.xzyw*sh.xxyy ;
    vec4 a1 = b1.xzyw + s1.xzyw*sh.zzww ;
    
    vec3 p0 = vec3(a0.xy,h.x);
    vec3 p1 = vec3(a0.zw,h.y);
    vec3 p2 = vec3(a1.xy,h.z);
    vec3 p3 = vec3(a1.zw,h.w);
    
    //Normalise gradients
    vec4 norm = taylorInvSqrt(vec4(dot(p0,p0), dot(p1,p1), dot(p2, p2), dot(p3,p3)));
    p0 *= norm.x;
    p1 *= norm.y;
    p2 *= norm.z;
    p3 *= norm.w;
    
    // Mix final noise value
    vec4 m = max(0.6 - vec4(dot(x0,x0), dot(x1,x1), dot(x2,x2), dot(x3,x3)), 0.0);
    m = m * m;
    return 42.0 * dot( m*m, vec4( dot(p0,x0), dot(p1,x1),
                                 dot(p2,x2), dot(p3,x3) ) );
}




//
// Description : Array and textureless GLSL 2D simplex noise function.
//      Author : Ian McEwan, Ashima Arts.
//  Maintainer : ijm
//     Lastmod : 20110822 (ijm)
//     License : Copyright (C) 2011 Ashima Arts. All rights reserved.
//               Distributed under the MIT License. See LICENSE file.
//               https://github.com/ashima/webgl-noise
//
vec2 mod289(vec2 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec3 permute(vec3 x) {
    return mod289(((x*34.0)+1.0)*x);
}

float snoise(vec2 v)
{
    const vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
                        0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
                        -0.577350269189626,  // -1.0 + 2.0 * C.x
                        0.024390243902439); // 1.0 / 41.0
    // First corner
    vec2 i  = floor(v + dot(v, C.yy) );
    vec2 x0 = v -   i + dot(i, C.xx);
    
    // Other corners
    vec2 i1;
    //i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
    //i1.y = 1.0 - i1.x;
    i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
    // x0 = x0 - 0.0 + 0.0 * C.xx ;
    // x1 = x0 - i1 + 1.0 * C.xx ;
    // x2 = x0 - 1.0 + 2.0 * C.xx ;
    vec4 x12 = x0.xyxy + C.xxzz;
    x12.xy -= i1;
    
    // Permutations
    i = mod289(i); // Avoid truncation effects in permutation
    vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
                     + i.x + vec3(0.0, i1.x, 1.0 ));
    
    vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
    m = m*m ;
    m = m*m ;
    
    // Gradients: 41 points uniformly over a line, mapped onto a diamond.
    // The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)
    
    vec3 x = 2.0 * fract(p * C.www) - 1.0;
    vec3 h = abs(x) - 0.5;
    vec3 ox = floor(x + 0.5);
    vec3 a0 = x - ox;
    
    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );
    
    // Compute final noise value at P
    vec3 g;
    g.x  = a0.x  * x0.x  + h.x  * x0.y;
    g.yz = a0.yz * x12.xz + h.yz * x12.yw;
    return 130.0 * dot(m, g);
}

//////////////////////// END INCLUDED /////////////////